#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Run.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "artdaq/DAQdata/PackageBuildInfo.hh"
#include "ds50daq/Version/GetReleaseVersion.h"

namespace ds50 {

  class DS50DaqBuildInfo : public art::EDProducer {
  public:
    explicit DS50DaqBuildInfo(fhicl::ParameterSet const & p);
    virtual ~DS50DaqBuildInfo() {}

    void beginRun(art::Run & r) override;
    void produce(art::Event & e) override;

  private:
    std::string const inst_name_;
    art::RunNumber_t current_run_;
  };

  DS50DaqBuildInfo::DS50DaqBuildInfo(fhicl::ParameterSet const &p):
    inst_name_(p.get<std::string>("instance_name", "buildinfo")),
    current_run_(0)
  {
    produces<artdaq::PackageBuildInfo, art::InRun>(inst_name_);
  }

  void DS50DaqBuildInfo::beginRun(art::Run &e) { 
    if (e.run () == current_run_) return;
    current_run_ = e.run ();

    // create a PackageBuildInfo object and add it to the Run
    std::unique_ptr<artdaq::PackageBuildInfo> build_info(new artdaq::PackageBuildInfo());
    std::string s1(ds50::getReleaseVersion());
    build_info->setPackageVersion(s1);
    std::string s2(ds50::getBuildDateTime());
    build_info->setBuildTimestamp(s2);

    e.put(std::move(build_info),inst_name_);
  }

  void DS50DaqBuildInfo::produce(art::Event &)
  {
    // nothing to be done for individual events
  }

  DEFINE_ART_MODULE(DS50DaqBuildInfo)
}
